import React, {useCallback, useContext, useEffect, useMemo, useRef, useState} from "react";

export let ContextParent = React.createContext({})

// 函数式组件
// 1. 没有生命周期
// 2. 没有 this
// 3. 使用各种 Hook 完成操作
// 4. 函数式组件本身相当于 render 函数
// 5. 函数式组件的第一个参数为 props
export default function SonComponent2(props= {
    msg:"",
    changeMsg:(_)=>{}
}){

    // 模拟 componentDidMount
    useEffect(() => {
        // 在组件挂载后执行的操作
        console.log('[挂载] SonComponent2 组件后执行的操作');
        return () => {
            // 在组件卸载前执行的操作
            console.log('[卸载] SonComponent2 组件 前执行的操作');
        };
    }, []); // 空数组意味着仅在组件挂载时执行一次

    const [state, setState] = useState({
        msg: "son msg",
        tmp: "",
        count: 0,
    })

    // 类似 vue computed, 缓存运算结果
    // 如果依赖项更新, 则重新计算
    // 组件初始化时 不会 执行
    let countMemo = useMemo(() => {
        console.log("state.count", state.count)
        return state.count
    }, [state.count]);

    // 模拟 componentDidUpdate
    // 类似 vue 的 watch
    // 组件初始化时 就会 执行
    useEffect(() => {
        // 当count更新时执行的操作
        console.log('SonComponent2 组件后更新，执行的操作:', state);
    }, [state]); // 依赖于 state，当 state 变化时执行

    function changeMsg(){
        let target = `parent msg 被修改 (${++state.count})`
        setState({
            ...state,
            tmp: props.changeMsg(target)
        })
    }

    // 缓存函数
    // 用于优化, 避免更新时重复创建
    const revokeChangeMsg = useCallback(() => {
        let count = state.count
        setState({
            ...state,
            count: count > 0 ? --count : 0,
            tmp: props.changeMsg(state.tmp)
        })
    }, [props, state]);


    let ref = useRef()
    useEffect(() => {
        console.log("SonComponent2 ref", ref, ref.current)
    }, []);

    return <div style={{border: '1px solid', width: '80%', overflow: 'auto'}}>
        子组件2

        <div ref={ref} style={{border: '1px solid', width: '80%', overflow: 'auto', margin: 'auto'}}>
            <div>
                {state.msg}
            </div>

            <hr/>

            <ContextParent.Provider value={{changeText: "修改msg", revokeText: "撤销msg"}}>
                <div>
                    {props.msg}
                    <div>count: {countMemo}</div>
                    <Consumer changeMsg={changeMsg} revokeChangeMsg={revokeChangeMsg}/>
                </div>
            </ContextParent.Provider>
        </div>
    </div>

}


function Consumer(props){
    let contextValue = useContext(ContextParent)
    console.log("SonComponent2 Consumer contextValue", contextValue)

    return <div>
        <button onClick={props.changeMsg}>{contextValue.changeText}</button>
        <button onClick={props.revokeChangeMsg}>{contextValue.revokeText}</button>
    </div>
}
